<?php

ini_set('display_errors', 'On');
error_reporting(E_ALL);
date_default_timezone_set('Pacific/Auckland');

require __DIR__.'/../../DEV/MChart/MChart.php';

class FuelHistory {
	
	protected $points = [];
	protected $times  = [];
	protected $rates  = [];
	protected $rates2 = [];
	
	protected $FIRST_ODO = null;
	protected $FIRST_TIME = null;
	
	public function __construct() {
	}
	
	public function loadDataFromFile($filename) {
		$this->loadData( explode("\r\n", file_get_contents($filename)) );
	}
	
	public function loadData($lines) {
		$totalfuel = null;
		$lastodo = 0;
		$running_rate = null;
				
		foreach($lines as $line) {
			if (!strlen($line) || $line[0] == '#' || $line[0] == ';') {
				continue;
			}
			
			list($ymd, $litres, $odo, $filled) = explode("\t", $line);
			
			//
			
			if (is_null($this->FIRST_ODO)) {
				$this->FIRST_ODO  = floor($odo / 1000) * 1000;
				$this->FIRST_TIME = strtotime($ymd)/86400;
			}
			
			// litres/km chart
			
			if ($litres == '?') {
				$totalfuel = 0;
			} else {
				$totalfuel += $litres;
			}
			
			if ($odo == '?') {
				
				$totalfuel = 0;
				$this->points[] = [$lastodo, 0];
				$lastodo = null;
				
			} else {		
				$lastodo = $odo - $this->FIRST_ODO;
				$this->points[] = [$lastodo, $totalfuel];
				
			}
			
			// odo/time chart
			
			if ($ymd != '?' && $odo != '?') {
				$this->times[] = [strtotime($ymd)/86400 - $this->FIRST_TIME, $odo - $this->FIRST_ODO];
			}
			
			// consumption rate
			
			if ($litres == '?' || $odo == '?') {
				$running_rate = null; // can't do anything with this data point
				
			} else if ($filled == 'Y') {
				
				if (! is_null($running_rate)) {
					// Used liters: $litres + $running_rate[0]
					// Used k's:    $odo - $running_rate[1]
					// Date range:  $running_rate[2] ... $ymd
					
					$k_p_l = ($odo - $running_rate[1]) / ($litres + $running_rate[0]);
					$l_p_100k = 100 / $k_p_l;
					
					$this->rates[]  = [strtotime($running_rate[2])/86400 - $this->FIRST_TIME, $k_p_l];
					$this->rates[]  = [strtotime($ymd            )/86400 - $this->FIRST_TIME, $k_p_l];
					$this->rates2[] = [strtotime($running_rate[2])/86400 - $this->FIRST_TIME, $l_p_100k];
					$this->rates2[] = [strtotime($ymd            )/86400 - $this->FIRST_TIME, $l_p_100k];
				}
				
				$running_rate = [0, $odo, $ymd, null];
				
			} else { // filled = N
				// add to running_rate
				if (! is_null($running_rate)) {
					$running_rate[0] += $litres;
				}
				
			}
			
		}
	}
	
	protected function getSimpleAxisRenderer() {
		return function($v) {
			return $v;
		};
	}
	
	protected function getDateAxisRenderer() {
		$first_time = $this->FIRST_TIME;
		$seen = [];
		return function($v) use (&$seen, $first_time) {
			$time = ($v + $first_time)*86400;
			
			$d = date('d', $time);
			$ym = date('Y-m', $time);
			
			if ($d == '01' && !array_key_exists($ym, $seen)) {
				$seen[$ym] = true;
				return $ym;
			} else {
				return '';
			}
		};
	}
	
	protected function getOdoRenderer() {
		$first_odo = $this->FIRST_ODO;
		return function($v) use ($first_odo) {
			return ($v % 1000) ? '' : $v + $first_odo;
		};
	}
	
	public function getKPLChart() {				
		$graph = new \mchart\LineChart();
		$graph->addSeries($this->rates, 'Km/Litre (higher is better)');
		$graph->xaxisrenderer = $this->getDateAxisRenderer();
		$graph->yaxisrenderer = $this->getSimpleAxisRenderer();
		return $graph;
	}
	
	public function getLP100KChart() {		
		$graph = new \mchart\LineChart();
		$graph->addSeries($this->rates2, 'Litres/100km (lower is better)');
		$graph->xaxisrenderer = $this->getDateAxisRenderer();
		$graph->yaxisrenderer = $this->getSimpleAxisRenderer();
		return $graph;
	}
	
	public function getFuelChart() {				
		$chart = new \mchart\StaggeredLineChart();
		$chart->addSeries($this->points, 'Litres/Km');
		$chart->xaxisrenderer = $this->getOdoRenderer();
		$chart->yaxisrenderer = function($v) {
			return ($v % 40) ? '' : $v.' L';
		};
		return $chart;
	}
	
	public function getOdoTimeChart() {		
		$graph = new \mchart\LineChart();
		$graph->addSeries($this->times, 'Km/Time');
		$graph->xaxisrenderer = $this->getDateAxisRenderer();
		$graph->yaxisrenderer = $this->getOdoRenderer();
		return $graph;
	}
}

// 

$fh = new FuelHistory();
$fh->loadDataFromFile('FuelHistory.tsv');

file_put_contents(__DIR__.'/'.date('YmdHis').'_KPLChart.png',    $fh->getKPLChart()    ->render(1280, 600, 1, true));
file_put_contents(__DIR__.'/'.date('YmdHis').'_LP100KChart.png', $fh->getLP100KChart() ->render(1280, 600, 1, true));
file_put_contents(__DIR__.'/'.date('YmdHis').'_FuelChart.png',   $fh->getFuelChart()   ->render(1280, 600, 1, true));
file_put_contents(__DIR__.'/'.date('YmdHis').'_OdoTime.png',     $fh->getOdoTimeChart()->render(1280, 600, 1, true));
